@@ -72,5 +72,12 @@
 
 #define _PKCS11H_ASSERT		assert
 
+#ifndef FALSE
+#define FALSE 0
+#endif
+#ifndef TRUE
+#define TRUE 1
+#endif
+
 #endif
 
