@@ -235,9 +235,13 @@ DSA_meth_free (DSA_METHOD *meth)
 static int
 DSA_meth_set1_name (DSA_METHOD *meth, const char *name)
 {
+#ifdef HAVE_DSA_METHOD_NAME
 	CK_RV rv;
 	rv = _pkcs11h_mem_strdup ((void *)&meth->name, name);
 	return rv == CKR_OK ? 1 : 0;
+#else
+	return 0;
+#endif
 }
 #endif
 
@@ -336,7 +340,7 @@ int
 __pkcs11h_openssl_ex_data_dup (
 	CRYPTO_EX_DATA *to,
 	CRYPTO_EX_DATA *from,
-	void *from_d,
+	void **from_d,
 	int idx,
 	long argl,
 	void *argp
@@ -346,7 +350,7 @@ int
 __pkcs11h_openssl_ex_data_dup (
 	CRYPTO_EX_DATA *to,
 	const CRYPTO_EX_DATA *from,
-	void *from_d,
+	void **from_d,
 	int idx,
 	long argl,
 	void *argp
@@ -474,9 +478,6 @@ __pkcs11h_openssl_rsa_dec (
 		case RSA_PKCS1_OAEP_PADDING:
 			mech = CKM_RSA_PKCS_OAEP;
 		break;
-		case RSA_SSLV23_PADDING:
-			rv = CKR_MECHANISM_INVALID;
-		break;
 		case RSA_NO_PADDING:
 			mech = CKM_RSA_X_509;
 		break;
@@ -656,6 +657,10 @@ __pkcs11h_openssl_session_setRSA(
 #if OPENSSL_VERSION_NUMBER < 0x10100001L
 	rsa->flags |= RSA_FLAG_SIGN_VER;
 #endif
+	if (EVP_PKEY_set1_RSA (evp, rsa) != 1) {
+		_PKCS11H_LOG (PKCS11H_LOG_WARN, "PKCS#11: Cannot set RSA key");
+		goto cleanup;
+	}
 
 #ifdef BROKEN_OPENSSL_ENGINE
 	if (!rsa->engine) {
@@ -713,6 +718,7 @@ __pkcs11h_openssl_dsa_do_sign(
 	OUT DSA *dsa
 ) {
 	pkcs11h_certificate_t certificate = __pkcs11h_openssl_dsa_get_pkcs11h_certificate (dsa);
+	PKCS11H_BOOL session_locked = FALSE;
 	unsigned char *sigbuf = NULL;
 	size_t siglen;
 	DSA_SIG *sig = NULL;
@@ -733,6 +739,11 @@ __pkcs11h_openssl_dsa_do_sign(
 	_PKCS11H_ASSERT (dsa!=NULL);
 	_PKCS11H_ASSERT (certificate!=NULL);
 
+	if ((rv = pkcs11h_certificate_lockSession (certificate)) != CKR_OK) {
+		goto cleanup;
+	}
+	session_locked = TRUE;
+
 	if (
 		(rv = pkcs11h_certificate_signAny (
 			certificate,
@@ -789,6 +800,11 @@ __pkcs11h_openssl_dsa_do_sign(
 
 cleanup:
 
+	if (session_locked) {
+		pkcs11h_certificate_releaseSession (certificate);
+		session_locked = FALSE;
+	}
+
 	if (sigbuf != NULL) {
 		_pkcs11h_mem_free ((void *)&sigbuf);
 	}
@@ -841,6 +857,11 @@ __pkcs11h_openssl_session_setDSA(
 	DSA_set_method (dsa, __openssl_methods.dsa);
 	DSA_set_ex_data (dsa, __openssl_methods.dsa_index, openssl_session);
 
+	if (EVP_PKEY_set1_DSA (evp, dsa) != 1) {
+		_PKCS11H_LOG (PKCS11H_LOG_WARN, "PKCS#11: Cannot set DSA key");
+		goto cleanup;
+	}
+
 	ret = TRUE;
 
 cleanup:
@@ -890,6 +911,7 @@ __pkcs11h_openssl_eckey_do_sign(
 	OUT EC_KEY *ec
 ) {
 	pkcs11h_certificate_t certificate = __pkcs11h_openssl_eckey_get_pkcs11h_certificate (ec);
+	PKCS11H_BOOL session_locked = FALSE;
 	unsigned char *sigbuf = NULL;
 	size_t siglen;
 	ECDSA_SIG *sig = NULL;
@@ -914,6 +936,11 @@ __pkcs11h_openssl_eckey_do_sign(
 	_PKCS11H_ASSERT (ec!=NULL);
 	_PKCS11H_ASSERT (certificate!=NULL);
 
+	if ((rv = pkcs11h_certificate_lockSession (certificate)) != CKR_OK) {
+		goto cleanup;
+	}
+	session_locked = TRUE;
+
 	if (
 		(rv = pkcs11h_certificate_signAny (
 			certificate,
@@ -974,6 +1001,11 @@ __pkcs11h_openssl_eckey_do_sign(
 
 cleanup:
 
+	if (session_locked) {
+		pkcs11h_certificate_releaseSession (certificate);
+		session_locked = FALSE;
+	}
+
 	if (sigbuf != NULL) {
 		_pkcs11h_mem_free ((void *)&sigbuf);
 	}
@@ -1028,6 +1060,11 @@ __pkcs11h_openssl_session_setECDSA(
 	EC_KEY_set_method (ec, __openssl_methods.eckey);
 	EC_KEY_set_ex_data (ec, __openssl_methods.eckey_index, openssl_session);
 
+	if (EVP_PKEY_set1_EC_KEY (evp, ec) != 1) {
+		_PKCS11H_LOG (PKCS11H_LOG_WARN, "PKCS#11: Cannot set EC key");
+		goto cleanup;
+	}
+
 	ret = TRUE;
 
 cleanup:
@@ -1465,7 +1502,7 @@ pkcs11h_openssl_session_getEVP (
 		}
 	}
 #endif
-#ifndef OPENSSL_NO_RSA
+#ifndef OPENSSL_NO_DSA
 	else if (EVP_PKEY_id (evp) == EVP_PKEY_DSA) {
 		if (!__pkcs11h_openssl_session_setDSA(openssl_session, evp)) {
 			goto cleanup;
