@@ -47,13 +47,13 @@ int main () {
 
 	if (
 		(rv = pkcs11h_addProvider (
+			"reference1",
 			TEST_PROVIDER,
-			TEST_PROVIDER,
-			FALSE,
-			PKCS11H_PRIVATEMODE_MASK_AUTO,
-			PKCS11H_SLOTEVENT_METHOD_AUTO,
-			0,
-			FALSE
+			TRUE,
+			PKCS11H_PRIVATEMODE_MASK_DECRYPT,
+			PKCS11H_SLOTEVENT_METHOD_POLL,
+			0x55,
+			TRUE
 		)) != CKR_OK
 	) {
 		fatal ("pkcs11h_addProvider failed", rv);
